#include "DOMTreeNode.h"

/* xml */



@implementation DOMTreeNode
@synthesize parent;
@synthesize children;
@synthesize key;
@synthesize leafvalue;

// Initialize all nodes as branches
- (DOMTreeNode *) init {
	if (self = [super init]) self.leafvalue = NULL;
	return self;
}

// Determine whether the node is a leaf or a branch
- (BOOL) isLeaf { return (self.leafvalue != NULL); }

// Return the first child that matches the key
	- (DOMTreeNode *) objectForKey: (NSString *) aKey {
		for (DOMTreeNode *node in self.children) 
			if ([[node key] isEqualToString: aKey]) return node;
		return NULL;
	}

// Return the last child leaf value that matches the key
	- (NSString *) leafForKey: (NSString *) aKey {
		for (DOMTreeNode *node in self.children) 
			if ([[node key] isEqualToString: aKey]) return node.leafvalue;
		return NULL;
	}

// Print the tree to standard out
- (void) dumpAtIndent: (int) indent {
	for (int i = 0; i < indent; i++) printf(" ");

	printf("[%2d] Key: %s ", indent, [self.key UTF8String]);
	if (leafvalue) printf("(%s)", [self.leafvalue UTF8String]);
	printf("\n");

	for (DOMTreeNode *node in self.children) [node dumpAtIndent:indent + 1];
}

- (void) dump { [self dumpAtIndent:0]; }

- (void) dealloc {
	if (self.parent) [self.parent release];
	if (self.key) [self.key release];
	if (self.leafvalue) [self.leafvalue release];
	if (self.children) [self.children release];
	[super dealloc];
}

@end


@implementation XMLParser
@synthesize root;
@synthesize stack;

// Public parser returns the tree root
- (DOMTreeNode *)parseXMLFile: (NSURL *) url {
	// Create a new clean stack
	self.stack = [[[NSMutableArray alloc] init] autorelease];

	// Initialize the root
	self.root = [[[DOMTreeNode alloc] init] autorelease];
	self.root.parent = NULL;
	self.root.leafvalue = NULL;
	self.root.children = [[[NSMutableArray alloc] init] autorelease];
	[self.stack addObject:self.root];

	// Parse the incoming data
	NSXMLParser *parser = [[[NSXMLParser alloc] initWithContentsOfURL:url] autorelease];
	[parser setDelegate:self];
	[parser parse];

	// pop down to real root
	return [self.root.children lastObject];
}

- (void) dealloc {
	if (self.stack) [self.stack release];
	if (self.root) [self.root release];
	[super dealloc];
}

// Descend to a new element
- (void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict {
	DOMTreeNode *leaf = [[[DOMTreeNode alloc] init] autorelease];
	leaf.parent = [stack lastObject];
	[leaf.parent.children addObject:leaf];
	[self.stack addObject:leaf];

	leaf.key = qName ? qName : elementName;
	[qName ? qName : elementName release];
	leaf.leafvalue = NULL;
	leaf.children = [[[NSMutableArray alloc] init] autorelease];

}

// Pop after finishing element
- (void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName {
	[self.stack removeLastObject];
}

// Reached a leaf
- (void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string {
	if (![[self.stack lastObject] leafvalue]) {
		[[self.stack lastObject] setLeafvalue:string];
		return;
	}
	[[self.stack lastObject] setLeafvalue:[NSString stringWithFormat:@"%@%@", [[self.stack lastObject] leafvalue], string]];
}
@end
